package src

import (
	"acme-mana/src/crypto"
	"bufio"
	"fmt"
	"github.com/go-acme/lego/v4/log"
	"github.com/go-acme/lego/v4/platform/config/env"
	"gopkg.in/yaml.v3"
	"os"
	"strings"
)

func ReadConfig() *AppConfig {
	if DnsProviderSupports != nil {
		return nil
	}
	InitConfig()
	file, err := os.ReadFile(GetEnvConf().ConfFile)
	if err != nil {
		log.Fatal(err)
	}
	var conf *AppConfig
	err = yaml.Unmarshal(file, &conf)
	if err != nil {
		log.Fatal(err)
	}
	return conf
}

func InitConfig() {
	// 判断当前目录下是否存在配置文件 config.yml
	_, err := os.Stat(GetEnvConf().ConfFile)

	if os.IsNotExist(err) {
		// 配置文件不存在，则创建一个
		log.Infof("配置文件不存在，自动创建默认配置文件")
		// 生成默认配置
		conf := readNewConf()
		// 创建一个默认的配置文件
		file, err := os.Create(GetEnvConf().ConfFile)
		if err != nil {
			log.Fatal(err)
		}
		data, err := yaml.Marshal(conf)
		if err != nil {
			log.Fatal(err)
		}
		// 写入配置文件
		_, err = file.Write(data)
		if err != nil {
			log.Fatal(err)
		}
	}

}

func readNewConf() *AppConfig {
	conf := defaultConf()
	log.Println("无配置文件, 生成配置文件")

	conf.CertDir = scanConfDefault("请输入证书保存目录; 默认为 cert", "cert")
	isGenMsg := "是否需要自动生成用于数据传输加密的RSA密钥对? \n请输入yes(Y)/no(N) 默认为 yes"
	isGenErrMsg := "请输入yes(Y)/no(N)"
	isGenValues := []string{"yes", "no", "Y", "N"}
	isGenEncrypt := scanConfDefaultCheck(isGenMsg, "yes", isGenValues, isGenErrMsg)
	if isGenEncrypt == "no" || isGenEncrypt == "N" {
		conf.Encrypt.PriKey = scanConf("请输入RSA私钥", "请输入RSA私钥")
		conf.Encrypt.PubKey = scanConf("请输入RSA公钥", "请输入RSA公钥")
	}

	msg := fmt.Sprintf("请输入DNS提供商; 当前支持的: %s", strings.Join(DnsProviderSupports, ","))
	errMsg := fmt.Sprintf("不支持的DNS提供商; 当前支持的: %s", strings.Join(DnsProviderSupports, ","))
	conf.Use = scanConfDefaultCheck(msg, "", DnsProviderSupports, errMsg)
	switch conf.Use {
	case "alidns":
		fmt.Printf("阿里云DNS配置帮助页: \n%s\n", "https://go-acme.github.io/lego/dns/alidns/index.html")
		fmt.Printf("阿里云令牌获取方式:\n%s\n", "https://usercenter.console.aliyun.com/#/manage/ak")
		fmt.Printf("阿里云SDK客户端项目地址:\n%s\n", "https://github.com/aliyun/alibaba-cloud-sdk-go?tab=readme-ov-file")
		conf.Provider.Ali.RegionID = scanConf("请输入阿里云Region ID", "请输入阿里云Region ID")
		conf.Provider.Ali.APIKey = scanConf("请输入阿里云API Key", "请输入阿里云API Key")
		conf.Provider.Ali.SecretKey = scanConf("请输入阿里云Secret Key", "请输入阿里云Secret Key")
	case "tencentcloud":
		fmt.Printf("腾讯云DNS配置帮助页: \n%s\n", "https://go-acme.github.io/lego/dns/tencentcloud/index.html")
		fmt.Printf("腾讯云令牌获取方式: \n%s\n", "https://console.cloud.tencent.com/cam/capi")
		fmt.Printf("腾讯云SDK客户端项目地址: \n%s\n", "https://github.com/tencentcloud/tencentcloud-sdk-go?tab=readme-ov-file")
		conf.Provider.Tencent.SecretId = scanConf("请输入腾讯云Secret Id", "请输入腾讯云Secret Id")
		conf.Provider.Tencent.SecretKey = scanConf("请输入腾讯云Secret Key", "请输入腾讯云Secret Key")
	case "cloudflare":
		fmt.Printf("Cloudflare DNS配置帮助页: \n%s\n", "https://go-acme.github.io/lego/dns/tencentcloud/index.html")
		fmt.Printf("Cloudflare 令牌获取方式: \n%s\n", "https://blog.cloudflare.com/zh-cn/api-tokens-general-availability/")
		fmt.Printf("Cloudflare SDK客户端项目地址: \n%s\n", "https://github.com/cloudflare/cloudflare-go")
		conf.Provider.CloudFlare.Token = scanConf("请输入CloudFlare DNS API Token", "请输入CloudFlare Token")
	}
	isAddDomainMsg := "是否需要添加证书获取配置?\n您可以在此处通过控制台交互添加;也可以在配置文件创建后,直接修改配置文件.\n请输入yes(Y)/no(N) 默认为 yes"
	isAddDomain := scanConfDefaultCheck(isAddDomainMsg, "yes", isGenValues, isGenErrMsg)
	if isAddDomain == "no" || isAddDomain == "N" {
		fmt.Printf("您可以通过手动修改%s文件, 调整证书获取配置.", GetEnvConf().ConfFile)
		conf.Domains = []Domain{}
		return conf
	}
	conf.Domains = []Domain{}
	for {
		name := scanConf("请输入配置名称", "配置名称不能为空")
		email := scanConf("请输入邮箱", "邮箱不能为空")
		host := scanConf("请输入主机名;支持泛解析;\n支持多个域名,多个域名用,(英文逗号)分割;\n如: example.com,*.example.com\n", "主机名不能为空")
		// 将host通过,分割为数组
		hosts := strings.Split(host, ",")
		conf.Domains = append(conf.Domains, Domain{
			Name:  name,
			Email: email,
			Host:  hosts,
		})
		isAddNextDomainMsg := "是否需要继续添加证书获取配置?\n请输入yes(Y)/no(N) 默认为 yes"
		isAddNextDomain := scanConfDefaultCheck(isAddNextDomainMsg, "yes", isGenValues, isGenErrMsg)
		if isAddNextDomain == "no" || isAddNextDomain == "N" {
			fmt.Printf("您后续可以通过手动修改%s文件, 调整证书获取配置.", GetEnvConf().ConfFile)
			return conf
		}
	}
}

// 读取用户输入
func scanConf(msg string, errMsg string) string {
	for {
		log.Println(msg)
		reader := bufio.NewReader(os.Stdin)
		name, err := reader.ReadString('\n')
		if err != nil {
			fmt.Println("读取失败;", err)
			continue
		}
		name = strings.Trim(name, "\r\n")
		if name == "" {
			fmt.Println(errMsg)
			continue
		}
		return name
	}
}

func scanConfDefault(msg string, defaultContent string) string {
	for {
		log.Println(msg)
		reader := bufio.NewReader(os.Stdin)
		name, err := reader.ReadString('\n')
		if err != nil {
			fmt.Println("读取失败;", err)
			continue
		}
		name = strings.Trim(name, "\r\n")
		if name == "" {
			return defaultContent
		}
		return name
	}
}

func scanConfDefaultCheck(msg string, defaultContent string, values []string, errMsg string) string {
	for {
		content := scanConfDefault(msg, defaultContent)
		// 判断内容是否在values中
		for _, value := range values {
			if value == content {
				return content
			}
		}
		log.Println(errMsg)
	}
}

func defaultConf() *AppConfig {
	//priKey, pubKey, err := GenRsa()
	priKey, pubKey, err := crypto.GenRSA()
	if err != nil {
		log.Fatal(err)
	}
	return &AppConfig{
		Use:     "Ali",
		CertDir: "cert",
		Provider: AppProvider{
			Ali: AliProvider{
				RegionID:  "cn-hangzhou",
				APIKey:    "api_key",
				SecretKey: "secret_key",
			},
			Tencent: TencentProvider{
				SecretId:  "secret_id",
				SecretKey: "secret_key",
			},
			CloudFlare: CloudFlareProvider{
				Token: "token",
			},
		},
		Domains: []Domain{
			{
				Name:  "example.com",
				Email: "email@example.com",
				Host:  []string{"www.example.com"},
			},
		},
		Encrypt: Encrypt{
			PriKey: priKey,
			PubKey: pubKey,
		},
	}
}

type AppConfig struct {
	Use string

	CertDir string

	Provider AppProvider

	Domains []Domain

	Encrypt Encrypt
}

type AppProvider struct {
	Ali        AliProvider
	Tencent    TencentProvider
	CloudFlare CloudFlareProvider
}
type AliProvider struct {
	RegionID  string
	APIKey    string
	SecretKey string
}
type TencentProvider struct {
	SecretId  string
	SecretKey string
}
type CloudFlareProvider struct {
	Token string
}
type Domain struct {
	Name  string
	Email string
	Host  []string
}
type Encrypt struct {
	PriKey string
	PubKey string
}

func (conf AppConfig) FindDomain(name string) *Domain {
	for _, domain := range conf.Domains {
		if domain.Name == name {
			return &domain
		}
	}
	return nil
}

const ENV_CONF_FILE = "ACME_MANA_CONF_FILE"

func InitRuntimeConf() *EnvConf {
	return &EnvConf{
		ConfFile: env.GetOrDefaultString(ENV_CONF_FILE, "config.yml"),
	}
}

type EnvConf struct {
	ConfFile string
}
